@@ -30,13 +30,16 @@ DimensionSettings.getPreferencesCategory = function(appPreferences) {
 };
 
 DimensionSettings.dimx = [
-            // widget name, dxf variable, factor relative to dimtxt
-            ["DIMTXT", RS.DIMTXT, 1.0],
-            ["DIMEXE", RS.DIMEXE, 0.5],
-            ["DIMEXO", RS.DIMEXO, 0.25],
-            ["DIMGAP", RS.DIMGAP, 0.25],
-            ["DIMASZ", RS.DIMASZ, 1.0],
-            ["DIMSCALE", RS.DIMSCALE, undefined],
+            // widget name, dxf variable, factor relative to dimtxt, default value
+            ["DIMTXT", RS.DIMTXT, 1.0, undefined],
+            ["DIMEXE", RS.DIMEXE, 0.5, undefined],
+            ["DIMEXO", RS.DIMEXO, 0.25, undefined],
+            ["DIMGAP", RS.DIMGAP, 0.25, undefined],
+            ["DIMASZ", RS.DIMASZ, 1.0, undefined],
+            ["DIMSCALE", RS.DIMSCALE, undefined, undefined],
+            ["DIMDLI", RS.DIMDLI, 2.0, undefined],
+            ["DIMTIH", RS.DIMTIH, undefined, false],
+            ["DIMTAD", RS.DIMTAD, undefined, 1],
         ];
 
 /**
@@ -54,7 +57,7 @@ DimensionSettings.updateUnit = function(unit) {
     // (re-)init unit labels:
     var unitSymbol = "";
     unitSymbol = RUnit.unitToName(DimensionSettings.unit);
-    for (var i=1; i<=5; i++) {
+    for (var i=1; i<=6; i++) {
         var w = widgets["DimUnit" + i];
         if (isNull(w)) {
             continue;
@@ -91,6 +94,7 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     var wastz = widgets["AngularShowTrailingZeros"];
     var wap = widgets["AngularPrecision"];
     var wdf = widgets["DimensionFont"];
+    var wdtc = widgets["DimensionTextColor"];
     var wdp = widgets["DecimalPoint"];
     var wfg = widgets["FontGroup"];
     var wkp = widgets["KeepProportions"];
@@ -152,11 +156,7 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
 
     // init linear dimension format combo boxes:
     if (!isNull(wlf)) {
-        wlf.addItem(qsTr("Scientific"), RS.Scientific);
-        wlf.addItem(qsTr("Decimal"), RS.Decimal);
-        wlf.addItem(qsTr("Engineering"), RS.Engineering);
-        wlf.addItem(qsTr("Architectural"), RS.ArchitecturalStacked);
-        wlf.addItem(qsTr("Fractional"), RS.FractionalStacked);
+        WidgetFactory.initDimlunitCombo(wlf);
         var defaultFormat = RUnit.isMetric(unit) ? RS.Decimal : RS.FractionalStacked;
         wlf.currentIndex = wlf.findData(RSettings.getIntValue(settingsName + "/LinearFormat", defaultFormat));
         // unsupported:
@@ -188,11 +188,7 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
 
     // init angular dimension format combo boxes:
     if (!isNull(waf)) {
-        waf.addItem(qsTr("Decimal Degrees"), RS.DegreesDecimal);
-        waf.addItem(qsTr("Deg/min/sec"), RS.DegreesMinutesSeconds);
-        waf.addItem(qsTr("Gradians"), RS.Gradians);
-        waf.addItem(qsTr("Radians"), RS.Radians);
-        waf.addItem(qsTr("Surveyor's units"), RS.Surveyors);
+        WidgetFactory.initDimaunitCombo(waf);
         waf.currentIndex = waf.findData(RSettings.getIntValue(settingsName + "/AngularFormat", RS.DegreesDecimal));
 
         waf["activated(int)"].connect(function() {
@@ -216,12 +212,14 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     }
 
     if (isNull(document)) {
+        // defaults for new drawings:
         // update unit labels, preview:
         //DimensionSettings.updateUnit(unit);
         //DimensionSettings.updateLinearPrecision(widgets);
         //DimensionSettings.updateAngularPrecision(widgets);
 
         if (hasPlugin("DWG")) {
+            // font:
             if (!isNull(wdf)) {
                 wdf.setProperty("Loaded", true);
                 wdf.editable = false;
@@ -229,11 +227,28 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
                 var dimFont = RSettings.getStringValue(settingsName + "/DimensionFont", "Standard");
                 activateFont(wdf, dimFont.isEmpty() ? "Standard" : dimFont);
             }
+
+            // text color:
+            if (!isNull(wdtc)) {
+                wdtc.setProperty("Loaded", true);
+                var dimTextColor = RSettings.getColorValue(settingsName + "/DimensionTextColor", new RColor(RColor.ByBlock));
+                wdtc.setColor(dimTextColor);
+            }
         }
         else {
-            if (!isNull(wfg)) {
-                wfg.visible = false;
+//            if (!isNull(wfg)) {
+//                wfg.visible = false;
+//            }
+
+            var hiddenWidgetNames = [ "DIMTIH", "DIMTAD", "FontGroup" ];
+            for (var i=0; i<hiddenWidgetNames.length; i++) {
+                var hiddenWidgetName = hiddenWidgetNames[i];
+                if (isNull(widgets[hiddenWidgetName])) {
+                    continue;
+                }
+                widgets[hiddenWidgetName].visible = false;
             }
+
         }
 
         // other global preferences are initialized automatically
@@ -241,6 +256,7 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     }
 
     // init dimension settings from document:
+    var dimStyle = document.queryDimStyleDirect();
     var keepProportions = true;
     var dimtxt = 0.0;
     var w;
@@ -251,18 +267,33 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
             continue;
         }
 
-        var dimprop = document.getKnownVariable(item[1], 2.5*item[2]);
+        var dimprop = dimStyle.getVariant(item[1]);
+        //var dimprop = document.getKnownVariable(item[1], 2.5*item[2]);
         if (item[1]===RS.DIMTXT) {
             dimtxt = dimprop;
         }
         else {
             if (Math.abs(dimprop-dimtxt*item[2]) > RS.PointTolerance) {
+                // detect that proportions are not kept:
                 keepProportions = false;
             }
         }
         //w.defaultValue = [w.text, w.getDefaultUnit()];
-        w.defaultValue = w.text;
-        w.setValue(dimprop);
+        if (isOfType(w, QCheckBox)) {
+            w.defaultValue = w.checked;
+            if (item[1]===RS.DIMTAD) {
+                // DIMTAD is an int but shown as checkbox:
+                w.checked = dimStyle.getInt(item[1])===1;
+            }
+            else {
+                w.checked = dimStyle.getBool(item[1]);
+            }
+            // document.getKnownVariable(item[1], item[3]);
+        }
+        else {
+            w.defaultValue = w.text;
+            w.setValue(dimprop);
+        }
         w.setProperty("Loaded", true);
     }
 
@@ -273,9 +304,8 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     }
 
     // init dimension arrow type:
-    var dimblk = document.getKnownVariable(RS.DIMBLK, "");
-    var dimtsz = document.getKnownVariable(RS.DIMTSZ, 0.0);
-    if (dimblk.toUpperCase()==="ARCHTICK" || dimtsz > RS.PointTolerance) {
+    var dimtsz = dimStyle.getVariant(RS.DIMTSZ);
+    if (dimtsz > RS.PointTolerance) {
         if (!isNull(widgets["ArchitecturalTick"])) {
             widgets["ArchitecturalTick"].checked = true;
         }
@@ -293,7 +323,8 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     }
 
     // init linear dimension format:
-    var dimlunit = document.getKnownVariable(RS.DIMLUNIT, RS.Decimal);
+    //var dimlunit = document.getKnownVariable(RS.DIMLUNIT, RS.Decimal);
+    var dimlunit = dimStyle.getInt(RS.DIMLUNIT);
     //widgets["LinearFormat"].currentIndex = dimlunit-1;
     if (!isNull(wlf)) {
         wlf.currentIndex = wlf.findData(dimlunit);
@@ -306,14 +337,16 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     }
 
     // init linear dimension precision:
-    var dimdec = document.getKnownVariable(RS.DIMDEC, 4);
+    //var dimdec = document.getKnownVariable(RS.DIMDEC, 4);
+    var dimdec = dimStyle.getInt(RS.DIMDEC);
     if (!isNull(wlp)) {
         wlp.currentIndex = dimdec;
         wlp.setProperty("Loaded", true);
     }
 
     // init angular dimension format:
-    var dimaunit = document.getKnownVariable(RS.DIMAUNIT, RS.DegreesDecimal);
+    //var dimaunit = document.getKnownVariable(RS.DIMAUNIT, RS.DegreesDecimal);
+    var dimaunit = dimStyle.getInt(RS.DIMAUNIT);
     if (!isNull(waf)) {
         //waf.currentIndex = dimaunit;
         waf.currentIndex = widgets["AngularFormat"].findData(dimaunit);
@@ -322,19 +355,22 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     }
 
     // init angular dimension precision:
-    var dimadec = document.getKnownVariable(RS.DIMADEC, 0);
+    //var dimadec = document.getKnownVariable(RS.DIMADEC, 0);
+    var dimadec = dimStyle.getInt(RS.DIMADEC);
     if (!isNull(wap)) {
         wap.currentIndex = dimadec;
         wap.setProperty("Loaded", true);
     }
 
     // show leading / trailing zeroes:
-    var dimzin = document.getKnownVariable(RS.DIMZIN, 12);
+    //var dimzin = document.getKnownVariable(RS.DIMZIN, 12);
+    var dimzin = dimStyle.getInt(RS.DIMZIN);
     if (!isNull(wlstz)) {
         wlstz.checked = !((dimzin & 8) === 8);
         wlstz.setProperty("Loaded", true);
     }
-    var dimazin = document.getKnownVariable(RS.DIMAZIN, 3);
+    //var dimazin = document.getKnownVariable(RS.DIMAZIN, 3);
+    var dimazin = dimStyle.getInt(RS.DIMAZIN);
     if (!isNull(wastz)) {
         wastz.checked = !((dimazin & 2) === 2);
         wastz.setProperty("Loaded", true);
@@ -342,7 +378,8 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
 
     // decimal separator:
     if (!isNull(wdp)) {
-        var dimdsep = document.getKnownVariable(RS.DIMDSEP, '.');
+        //var dimdsep = document.getKnownVariable(RS.DIMDSEP, '.');
+        var dimdsep = dimStyle.getInt(RS.DIMDSEP);
         if (dimdsep===0) {
             dimdsep = '.'.charCodeAt(0);
         }
@@ -355,6 +392,7 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
     DimensionSettings.updateUnit(unit);
 
     if (hasPlugin("DWG")) {
+        // font:
         if (!isNull(wdf)) {
             wdf.setProperty("Loaded", true);
             wdf.editable = false;
@@ -363,6 +401,16 @@ DimensionSettings.initPreferences = function(pageWidget, calledByPrefDialog, doc
             var dimFont = document.getDimensionFont();
             activateFont(wdf, dimFont.isEmpty() ? "Standard" : dimFont);
         }
+
+        // text color:
+        if (!isNull(wdtc)) {
+            wdtc.setProperty("Loaded", true);
+            //var dimTextColor = document.getKnownVariable(RS.DIMCLRT, new RColor(RColor.ByBlock));
+            var dimTextColor = dimStyle.getColor(RS.DIMCLRT);
+            if (isValidColor(dimTextColor)) {
+                wdtc.setColor(dimTextColor);
+            }
+        }
     }
     else {
         if (!isNull(wfg)) {
@@ -641,50 +689,85 @@ DimensionSettings.savePreferences = function(pageWidget, calledByPrefDialog, doc
         return;
     }
 
+    var dimStyle = document.queryDimStyle();
+
     for (var i=0; i<DimensionSettings.dimx.length; i++) {
         var item = DimensionSettings.dimx[i];
-        document.setKnownVariable(item[1], widgets[item[0]].getValue(), transaction);
-        widgets[item[0]].setProperty("Saved", true);
+        var w = widgets[item[0]];
+        if (isOfType(w, QCheckBox)) {
+
+            dimStyle.setVariant(item[1], w.checked);
+//            if (item[1]===RS.DIMTAD) {
+//                //document.setKnownVariable(item[1], w.checked ? 1 : 0, transaction);
+//                qDebug("checkbox " + w.objectName + " is checked:" + w.checked);
+//            }
+//            else {
+//                dimStyle.setVariant(item[1], w.checked);
+//            }
+        }
+        else {
+            //document.setKnownVariable(item[1], w.getValue(), transaction);
+            dimStyle.setVariant(item[1], w.getValue());
+        }
+        w.setProperty("Saved", true);
     }
 
     // dimension arrow type:
     if (widgets["ArchitecturalTick"].checked) {
-        document.setKnownVariable(RS.DIMTSZ, document.getKnownVariable(RS.DIMASZ), transaction);
+        //dimStyle.setBool(RS.QCADARCHTICK, true);
+        dimStyle.setDouble(RS.DIMTSZ, dimStyle.getDouble(RS.DIMASZ));
+        //document.setKnownVariable(RS.DIMTSZ, document.getKnownVariable(RS.DIMASZ), transaction);
         //document.setKnownVariable(RS.DIMBLK, "ArchTick", transaction);
     }
     else {
-        document.setKnownVariable(RS.DIMTSZ, 0.0, transaction);
+        //dimStyle.setBool(RS.QCADARCHTICK, false);
+        dimStyle.setDouble(RS.DIMTSZ, 0.0);
+        //document.setKnownVariable(RS.DIMTSZ, 0.0, transaction);
         //document.setKnownVariable(RS.DIMBLK, "", transaction);
     }
+
     widgets["ArchitecturalTick"].setProperty("Saved", true);
     widgets["Arrow"].setProperty("Saved", true);
 
     // decimal point:
-    document.setKnownVariable(RS.DIMDSEP, widgets["DecimalPoint"].itemData(widgets["DecimalPoint"].currentIndex), transaction);
+    //document.setKnownVariable(RS.DIMDSEP, widgets["DecimalPoint"].itemData(widgets["DecimalPoint"].currentIndex), transaction);
+    dimStyle.setInt(RS.DIMDSEP, widgets["DecimalPoint"].itemData(widgets["DecimalPoint"].currentIndex));
 
     // dimension format / precision:
-    document.setKnownVariable(RS.DIMLUNIT, widgets["LinearFormat"].currentIndex+1, transaction);
-    document.setKnownVariable(RS.DIMDEC, widgets["LinearPrecision"].currentIndex, transaction);
-    document.setKnownVariable(RS.DIMAUNIT, widgets["AngularFormat"].currentIndex, transaction);
-    document.setKnownVariable(RS.DIMADEC, widgets["AngularPrecision"].currentIndex, transaction);
+    dimStyle.setInt(RS.DIMLUNIT, widgets["LinearFormat"].currentIndex+1);
+    dimStyle.setInt(RS.DIMDEC, widgets["LinearPrecision"].currentIndex);
+    dimStyle.setInt(RS.DIMAUNIT, widgets["AngularFormat"].currentIndex);
+    dimStyle.setInt(RS.DIMADEC, widgets["AngularPrecision"].currentIndex);
+    dimStyle.setColor(RS.DIMCLRT, widgets["DimensionTextColor"].getColor());
+
+//    document.setKnownVariable(RS.DIMLUNIT, widgets["LinearFormat"].currentIndex+1, transaction);
+//    document.setKnownVariable(RS.DIMDEC, widgets["LinearPrecision"].currentIndex, transaction);
+//    document.setKnownVariable(RS.DIMAUNIT, widgets["AngularFormat"].currentIndex, transaction);
+//    document.setKnownVariable(RS.DIMADEC, widgets["AngularPrecision"].currentIndex, transaction);
+//    document.setKnownVariable(RS.DIMCLRT, widgets["DimensionTextColor"].getColor(), transaction);
     document.setDimensionFont(widgets["DimensionFont"].currentText, transaction);
 
     // show leading / trailing zeroes:
     if (widgets["LinearShowTrailingZeros"].checked) {
-        document.setKnownVariable(RS.DIMZIN, 0, transaction);
+        dimStyle.setInt(RS.DIMZIN, 0);
+        //document.setKnownVariable(RS.DIMZIN, 0, transaction);
     }
     else {
-        document.setKnownVariable(RS.DIMZIN, 8, transaction);
+        dimStyle.setInt(RS.DIMZIN, 8);
+        //document.setKnownVariable(RS.DIMZIN, 8, transaction);
     }
     widgets["LinearShowTrailingZeros"].setProperty("Saved", true);
 
     if (widgets["AngularShowTrailingZeros"].checked) {
-        document.setKnownVariable(RS.DIMAZIN, 0, transaction);
+        dimStyle.setInt(RS.DIMAZIN, 0);
+        //document.setKnownVariable(RS.DIMAZIN, 0, transaction);
     }
     else {
-        document.setKnownVariable(RS.DIMAZIN, 2, transaction);
+        dimStyle.setInt(RS.DIMAZIN, 2);
+        //document.setKnownVariable(RS.DIMAZIN, 2, transaction);
     }
     widgets["AngularShowTrailingZeros"].setProperty("Saved", true);
+    transaction.addObject(dimStyle);
 
 
     // force update of bounding box of dimension entities:
